/*
 * Copyright (c) 2009 Mysema Ltd.
 * All rights reserved.
 * 
 */
package com.mysema.commons.fluxml;

import java.io.IOException;
import java.io.Writer;
import java.util.Map;
import java.util.Stack;

import org.apache.commons.lang.StringUtils;

/**
 * AbstractMarkupWriter is an abstract base class for markup writers
 * 
 * @author tiwe
 * @version $Id$
 */
public abstract class AbstractMarkupWriter<SubType extends AbstractMarkupWriter<SubType>>
        implements MarkupWriter<SubType> {

    private boolean open = false;

    private Writer writer;

    @SuppressWarnings("unchecked")
    private SubType _this = (SubType) this;

    private Stack<String> openElements = new Stack<String>();

    public AbstractMarkupWriter(Writer writer) {
        this.writer = writer;
    }

    public SubType attribute(String a, String val) throws IOException {        
        if (!StringUtils.isEmpty(val)){
            writer.write(" " + a + "=\"");
            escapeMarkup(writer, val);
            writer.write("\"");
        }                
        return _this;
    }

    public SubType attributes(Map<String, String> attributes) throws IOException {
        for (Map.Entry<String, String> entry : attributes.entrySet()) {
            attribute(entry.getKey(), entry.getValue());
        }
        return _this;
    }

    protected abstract void escapeMarkup(Writer writer, String val) throws IOException;

    public SubType begin(String name) throws IOException {
        if (open) {
            writer.write(">");
        }
        writer.write("<" + name);
        openElements.push(name);
        open = true;
        return _this;
    }

    public SubType end(String name) throws IOException {
        if (!openElements.peek().equals(name)) {
            throw new IllegalArgumentException("Element " + openElements.peek()
                    + " not closed");
        }
        if (open) {
            closeEmptyTag(writer);
            open = false;
        } else {
            writer.write("</" + name + ">");
        }
        openElements.pop();
        return _this;
    }

    protected abstract void closeEmptyTag(Writer writer) throws IOException;

    public SubType print(String text) throws IOException {
        return printInternal(text, true);
    }
    
    public SubType printRaw(String text) throws IOException {
        return printInternal(text, false);
    }
    
    private SubType printInternal(String text, boolean escape) throws IOException{
        if (open) {
            writer.write(">");
            open = false;
        }
        if (escape){
            escapeMarkup(writer, text);
        }else{
            writer.write(text);
        }
        return _this;
    }

}
